@@ -1,5 +1,5 @@
 package Catalyst::Action::Serialize;
-
+$Catalyst::Action::Serialize::VERSION = '1.17';
 use Moose;
 use namespace::autoclean;
 
@@ -7,8 +7,6 @@ extends 'Catalyst::Action::SerializeBase';
 use Module::Pluggable::Object;
 use MRO::Compat;
 
-our $VERSION = '1.16'; # VERSION
-
 has _encoders => (
    is => 'ro',
    isa => 'HashRef',
@@ -28,6 +26,15 @@ sub execute {
     return 1 if defined $c->stash->{current_view};
     return 1 if defined $c->stash->{current_view_instance};
 
+    # on 3xx responses, serialize if there's something to
+    # serialize, no-op if not
+    my $stash_key = (
+       $controller->{'serialize'} ?
+           $controller->{'serialize'}->{'stash_key'} :
+                $controller->{'stash_key'}
+           ) || 'rest';
+    return 1 if $c->response->status =~ /^(?:3\d\d)$/ && ! defined $c->stash->{$stash_key};
+
     my ( $sclass, $sarg, $content_type ) =
       $self->_load_content_plugins( "Catalyst::Action::Serialize",
         $controller, $c );
